@@ -49,7 +49,7 @@ struct spa_pod_builder {
 	struct spa_callbacks callbacks;
 };
 
-#define SPA_POD_BUILDER_INIT(buffer,size)  ((struct spa_pod_builder){ (buffer), (size), 0, {}, {} })
+#define SPA_POD_BUILDER_INIT(buffer,size)  ((struct spa_pod_builder){ (buffer), (size), 0, {0,0,NULL},{NULL,NULL}})
 
 static inline void
 spa_pod_builder_get_state(struct spa_pod_builder *builder, struct spa_pod_builder_state *state)
@@ -481,7 +481,7 @@ do {										\
 		spa_pod_builder_long(builder, va_arg(args, int64_t));		\
 		break;								\
 	case 'f':								\
-		spa_pod_builder_float(builder, va_arg(args, double));		\
+		spa_pod_builder_float(builder, (float)va_arg(args, double));	\
 		break;								\
 	case 'd':								\
 		spa_pod_builder_double(builder, va_arg(args, double));		\
