@@ -1,26 +1,6 @@
-/* Spa A2DP AAC codec
- *
- * Copyright © 2020 Wim Taymans
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* Spa A2DP AAC codec */
+/* SPDX-FileCopyrightText: Copyright © 2020 Wim Taymans */
+/* SPDX-License-Identifier: MIT */
 
 #include <unistd.h>
 #include <stddef.h>
@@ -37,9 +17,6 @@
 #include "media-codecs.h"
 
 static struct spa_log *log;
-static struct spa_log_topic log_topic = SPA_LOG_TOPIC(0, "spa.bluez5.codecs.aac");
-#undef SPA_LOG_TOPIC_DEFAULT
-#define SPA_LOG_TOPIC_DEFAULT &log_topic
 
 #define DEFAULT_AAC_BITRATE	320000
 #define MIN_AAC_BITRATE		64000
@@ -65,15 +42,41 @@ struct impl {
 	int samplesize;
 };
 
+static bool eld_supported(void)
+{
+	static bool supported = false, checked = false;
+	HANDLE_AACENCODER aacenc = NULL;
+
+	if (checked)
+		return supported;
+
+	if (aacEncOpen(&aacenc, 0, 2) != AACENC_OK)
+		goto done;
+	if (aacEncoder_SetParam(aacenc, AACENC_AOT, AOT_ER_AAC_ELD) != AACENC_OK)
+		goto done;
+	if (aacEncoder_SetParam(aacenc,  AACENC_SBR_MODE, 1) != AACENC_OK)
+		goto done;
+
+	supported = true;
+
+done:
+	if (aacenc)
+		aacEncClose(&aacenc);
+	checked = true;
+	spa_log_debug(log, "FDK-AAC AAC-ELD support:%d", (int)supported);
+	return supported;
+}
+
 static int codec_fill_caps(const struct media_codec *codec, uint32_t flags,
 		uint8_t caps[A2DP_MAX_CAPS_SIZE])
 {
-	static const a2dp_aac_t a2dp_aac = {
+	const a2dp_aac_t a2dp_aac = {
 		.object_type =
 			/* NOTE: AAC Long Term Prediction and AAC Scalable are
 			 *       not supported by the FDK-AAC library. */
 			AAC_OBJECT_TYPE_MPEG2_AAC_LC |
-			AAC_OBJECT_TYPE_MPEG4_AAC_LC,
+			AAC_OBJECT_TYPE_MPEG4_AAC_LC |
+			(eld_supported() ? AAC_OBJECT_TYPE_MPEG4_AAC_ELD : 0),
 		AAC_INIT_FREQUENCY(
 			AAC_SAMPLING_FREQ_8000 |
 			AAC_SAMPLING_FREQ_11025 |
@@ -143,16 +146,26 @@ static int codec_select_config(const struct media_codec *codec, uint32_t flags,
 
 	conf = *(a2dp_aac_t*)caps;
 
-	if (conf.object_type & AAC_OBJECT_TYPE_MPEG2_AAC_LC)
-		conf.object_type = AAC_OBJECT_TYPE_MPEG2_AAC_LC;
-	else if (conf.object_type & AAC_OBJECT_TYPE_MPEG4_AAC_LC)
-		conf.object_type = AAC_OBJECT_TYPE_MPEG4_AAC_LC;
-	else if (conf.object_type & AAC_OBJECT_TYPE_MPEG4_AAC_LTP)
-		return -ENOTSUP;  /* Not supported by FDK-AAC */
-	else if (conf.object_type & AAC_OBJECT_TYPE_MPEG4_AAC_SCA)
-		return -ENOTSUP;  /* Not supported by FDK-AAC */
-	else
-		return -ENOTSUP;
+	if (codec->id == SPA_BLUETOOTH_AUDIO_CODEC_AAC_ELD) {
+		if (!eld_supported())
+			return -ENOTSUP;
+			
+		if (conf.object_type & AAC_OBJECT_TYPE_MPEG4_AAC_ELD)
+			conf.object_type = AAC_OBJECT_TYPE_MPEG4_AAC_ELD;
+		else
+			return -ENOTSUP;
+	} else {
+		if (conf.object_type & AAC_OBJECT_TYPE_MPEG2_AAC_LC)
+			conf.object_type = AAC_OBJECT_TYPE_MPEG2_AAC_LC;
+		else if (conf.object_type & AAC_OBJECT_TYPE_MPEG4_AAC_LC)
+			conf.object_type = AAC_OBJECT_TYPE_MPEG4_AAC_LC;
+		else if (conf.object_type & AAC_OBJECT_TYPE_MPEG4_AAC_LTP)
+			return -ENOTSUP;  /* Not supported by FDK-AAC */
+		else if (conf.object_type & AAC_OBJECT_TYPE_MPEG4_AAC_SCA)
+			return -ENOTSUP;  /* Not supported by FDK-AAC */
+		else
+			return -ENOTSUP;
+	}
 
 	if ((i = media_codec_select_config(aac_frequencies,
 					  SPA_N_ELEMENTS(aac_frequencies),
@@ -213,12 +226,12 @@ static int codec_enum_config(const struct media_codec *codec, uint32_t flags,
 			spa_pod_builder_int(b, f->value);
 		}
 	}
-	if (i == 0)
-		return -EINVAL;
 	if (i > 1)
 		choice->body.type = SPA_CHOICE_Enum;
 	spa_pod_builder_pop(b, &f[1]);
 
+	if (i == 0)
+		return -EINVAL;
 
 	if (SPA_FLAG_IS_SET(conf.channels, AAC_CHANNELS_1 | AAC_CHANNELS_2)) {
 		spa_pod_builder_add(b,
@@ -270,7 +283,8 @@ static int codec_validate_config(const struct media_codec *codec, uint32_t flags
 	 * a BlueZ bug, but we can be lax here and below in codec_init.
 	 */
 	if (!(conf.object_type & (AAC_OBJECT_TYPE_MPEG2_AAC_LC |
-					AAC_OBJECT_TYPE_MPEG4_AAC_LC)))
+					AAC_OBJECT_TYPE_MPEG4_AAC_LC |
+					AAC_OBJECT_TYPE_MPEG4_AAC_ELD)))
 		return -EINVAL;
 	j = 0;
 	SPA_FOR_EACH_ELEMENT_VAR(aac_frequencies, f) {
@@ -350,16 +364,27 @@ static void *codec_init(const struct media_codec *codec, uint32_t flags,
 	if (res != AACENC_OK)
 		goto error;
 
-	if (!(conf->object_type & (AAC_OBJECT_TYPE_MPEG2_AAC_LC |
-					AAC_OBJECT_TYPE_MPEG4_AAC_LC))) {
+	/* If object type has multiple bits set (invalid per spec, see above),
+	 * assume the device usually means AAC-LC.
+	 */
+	if (conf->object_type & (AAC_OBJECT_TYPE_MPEG2_AAC_LC |
+						AAC_OBJECT_TYPE_MPEG4_AAC_LC)) {
+		res = aacEncoder_SetParam(this->aacenc, AACENC_AOT, AOT_AAC_LC);
+		if (res != AACENC_OK)
+			goto error;
+	} else if (conf->object_type & AAC_OBJECT_TYPE_MPEG4_AAC_ELD) {
+		res = aacEncoder_SetParam(this->aacenc, AACENC_AOT, AOT_ER_AAC_ELD);
+		if (res != AACENC_OK)
+			goto error;
+
+		res = aacEncoder_SetParam(this->aacenc,  AACENC_SBR_MODE, 1);
+		if (res != AACENC_OK)
+			goto error;
+	} else {		
 		res = -EINVAL;
 		goto error;
 	}
 
-	res = aacEncoder_SetParam(this->aacenc, AACENC_AOT, AOT_AAC_LC);
-	if (res != AACENC_OK)
-		goto error;
-
 	res = aacEncoder_SetParam(this->aacenc, AACENC_SAMPLERATE, this->rate);
 	if (res != AACENC_OK)
 		goto error;
@@ -628,7 +653,7 @@ static int codec_increase_bitpool(void *data)
 static void codec_set_log(struct spa_log *global_log)
 {
 	log = global_log;
-	spa_log_topic_init(log, &log_topic);
+	spa_log_topic_init(log, &codec_plugin_log_topic);
 }
 
 const struct media_codec a2dp_codec_aac = {
@@ -655,7 +680,33 @@ const struct media_codec a2dp_codec_aac = {
 	.set_log = codec_set_log,
 };
 
+const struct media_codec a2dp_codec_aac_eld = {
+	.id = SPA_BLUETOOTH_AUDIO_CODEC_AAC_ELD,
+	.codec_id = A2DP_CODEC_MPEG24,
+	.name = "aac_eld",
+	.description = "AAC-ELD",
+	.endpoint_name = "aac",
+	.fill_caps = NULL,
+	.select_config = codec_select_config,
+	.enum_config = codec_enum_config,
+	.validate_config = codec_validate_config,
+	.init_props = codec_init_props,
+	.clear_props = codec_clear_props,
+	.init = codec_init,
+	.deinit = codec_deinit,
+	.get_block_size = codec_get_block_size,
+	.start_encode = codec_start_encode,
+	.encode = codec_encode,
+	.start_decode = codec_start_decode,
+	.decode = codec_decode,
+	.abr_process = codec_abr_process,
+	.reduce_bitpool = codec_reduce_bitpool,
+	.increase_bitpool = codec_increase_bitpool,
+	.set_log = codec_set_log,
+};
+
 MEDIA_CODEC_EXPORT_DEF(
 	"aac",
-	&a2dp_codec_aac
+	&a2dp_codec_aac,
+	&a2dp_codec_aac_eld
 );
