@@ -252,13 +252,16 @@ enum spa_bt_hfp_ag_feature {
 };
 
 enum spa_bt_hfp_sdp_ag_features {
-	SPA_BT_HFP_SDP_AG_FEATURE_NONE =		(0),
-	SPA_BT_HFP_SDP_AG_FEATURE_3WAY =		(1 << 0),
-	SPA_BT_HFP_SDP_AG_FEATURE_ECNR =		(1 << 1),
-	SPA_BT_HFP_SDP_AG_FEATURE_VOICE_RECOG =		(1 << 2),
-	SPA_BT_HFP_SDP_AG_FEATURE_IN_BAND_RING_TONE =	(1 << 3),
-	SPA_BT_HFP_SDP_AG_FEATURE_ATTACH_VOICE_TAG =	(1 << 4),
-	SPA_BT_HFP_SDP_AG_FEATURE_WIDEBAND_SPEECH =	(1 << 5),
+	SPA_BT_HFP_SDP_AG_FEATURE_NONE =			(0),
+	SPA_BT_HFP_SDP_AG_FEATURE_3WAY =			(1 << 0),
+	SPA_BT_HFP_SDP_AG_FEATURE_ECNR =			(1 << 1),
+	SPA_BT_HFP_SDP_AG_FEATURE_VOICE_RECOG =			(1 << 2),
+	SPA_BT_HFP_SDP_AG_FEATURE_IN_BAND_RING_TONE =		(1 << 3),
+	SPA_BT_HFP_SDP_AG_FEATURE_ATTACH_VOICE_TAG =		(1 << 4),
+	SPA_BT_HFP_SDP_AG_FEATURE_WIDEBAND_SPEECH =		(1 << 5),
+	SPA_BT_HFP_SDP_AG_FEATURE_ENH_VOICE_RECOG_STATUS =	(1 << 6),
+	SPA_BT_HFP_SDP_AG_FEATURE_VOICE_RECOG_TEXT =		(1 << 7),
+	SPA_BT_HFP_SDP_AG_FEATURE_SUPER_WIDEBAND_SPEECH =	(1 << 8),
 };
 
 enum spa_bt_hfp_hf_feature {
@@ -299,13 +302,16 @@ enum spa_bt_hfp_hf_xapl_features {
 };
 
 enum spa_bt_hfp_sdp_hf_features {
-	SPA_BT_HFP_SDP_HF_FEATURE_NONE =		(0),
-	SPA_BT_HFP_SDP_HF_FEATURE_ECNR =		(1 << 0),
-	SPA_BT_HFP_SDP_HF_FEATURE_3WAY =		(1 << 1),
-	SPA_BT_HFP_SDP_HF_FEATURE_CLIP =		(1 << 2),
-	SPA_BT_HFP_SDP_HF_FEATURE_VOICE_RECOGNITION =	(1 << 3),
+	SPA_BT_HFP_SDP_HF_FEATURE_NONE =			(0),
+	SPA_BT_HFP_SDP_HF_FEATURE_ECNR =			(1 << 0),
+	SPA_BT_HFP_SDP_HF_FEATURE_3WAY =			(1 << 1),
+	SPA_BT_HFP_SDP_HF_FEATURE_CLIP =			(1 << 2),
+	SPA_BT_HFP_SDP_HF_FEATURE_VOICE_RECOGNITION =		(1 << 3),
 	SPA_BT_HFP_SDP_HF_FEATURE_REMOTE_VOLUME_CONTROL =	(1 << 4),
-	SPA_BT_HFP_SDP_HF_FEATURE_WIDEBAND_SPEECH =	(1 << 5),
+	SPA_BT_HFP_SDP_HF_FEATURE_WIDEBAND_SPEECH =		(1 << 5),
+	SPA_BT_HFP_SDP_HF_FEATURE_ENH_VOICE_RECOG_STATUS =	(1 << 6),
+	SPA_BT_HFP_SDP_HF_FEATURE_VOICE_RECOG_TEXT =		(1 << 7),
+	SPA_BT_HFP_SDP_HF_FEATURE_SUPER_WIDEBAND_SPEECH =	(1 << 8),
 };
 
 static inline const char *spa_bt_profile_name (enum spa_bt_profile profile) {
